﻿<#
.NOTES
	Name: Get-ExOTLSReport.ps1
	Author: Thomas Windscheif / excITe Consulting
	Requires:	PowerShell v3
				Remote-PowerShell-User at least needs the permission to access Cmdlets Get-MessageTrace & Get-MessageTraceDetail
	Homepage:	http://www.excite-consulting.de/
				https://www.microlinc.de/
	Version History:
	1.1 - 2019/03/05 - Added support for more than 1000 Messagetrace-Entries
	1.0 - 2019/02/15 - Initial Release
.SYNOPSIS
	This script generates a HTML and detailed CSV-report for sender- and recipient domainbased TLS-Sessions.
	The report gives an overview of the TLS Compliance.
.DESCRIPTION
	This script retrieved all receive and send-events of the message trace.
	The tracedetails are parsed and aggregated to a simple HTML-Report and a detailed CSV-based report.
	The script can send the reports via mail to specified recipient.
.PARAMETER InternalDomains
	This parameter defines the domains considered to be yours.
	e.g. "yourdomain.onmicrosoft.com","yourdomain.de"
.PARAMETER AmountofDaysToCheck
	Defines the numbers of days from now to the past, which should be analysed
.PARAMETER MailServer
	Specify the IP-address or FQDN of your Mailserver which relays the HTML-Report to your mailbox
.PARAMETER MailServerPort
	Specify the TCP-Port to be used when sending mails to your mail server
.PARAMETER MailServerUsername
	Specify Username for your mailserver which is used to send mails
.PARAMETER MailServerPassword
	Specify Password for your mailserver which is used to send mails
.PARAMETER MailServerUseTLS
	Specify whether the connection should enforce STARTTLS.
.PARAMETER MailServerIgnoreTLSValidation
	If you use untrusted self signed certificates on your mailserver you can choose to disable the certificate verification.
	NOT RECOMMENDED IN PRODUCTION USE.
.PARAMETER MessageSender
	Defines the Sender of the mails send by this script.
.PARAMETER MessageRecipient
	Defines the Recipient of the mails send by this script.
.PARAMETER MessageSubject
	Defines the message subject of the mails send by this script
.PARAMETER SendMail
	If set to true, the script sends an email to the specified recpients (MessageRecipient)
.PARAMETER CreateDetailCSV
	If set to true, the script creates and saves a detailed CSV-Report in the Logfolder specified with LogFilePath.
.PARAMETER FileNamePrefix
	Defines the Filenameprefix for CSV- and html-report-file created by the script
.PARAMETER LogFilePath
	Defines the LogfilePath in which the CSV, HTML and Script-Runtime-Log are saved
.PARAMETER ExchangeOnlineURL
	The URI für Exchange Online PowerShell-Access is specified here
	Default for Office 365 worldwide: http://outlook.office365.com/powershell-liveid
.PARAMETER ExOUsername
	Exchange Online Username with the permission to access Remote PowerShell
	e.g. svc-messagetrace@yourdomain.onmicrosoft.com
.PARAMETER ExOPassword
	Password of the User in Cleartext
.PARAMETER DisableDisplayLog
	Disables Output of Log-Entries on Standard Out
.PARAMETER DeleteItemsOlderThanDays
	HTML-, CSV- and Logfiles older than the defined Days are deleted automatically at the end of the script.
	If you set the value to 0, nothing will be deleted.
#>

[CmdletBinding()]
Param(
[array]$InternalDomains = @("yourdomain.onmicrosoft.com","yourdomain.de"),
[int]$AmountofDaysToCheck = 14,
$MailServer = "mail.yourdomain.de",
[int]$MailServerPort = 25,
[string]$MailServerUsername = "",
[string]$MailServerPassword = "",
[bool]$MailServerUseTLS = $true,
[bool]$MailServerIgnoreTLSValidation = $false,
[string]$MessageSender = "tlsreport@yourdomain.de",
[string]$MessageRecipient = "compliance@yourdomain.de",
[string]$MessageSubject = "Exchange Online TLS-Report",
[bool]$SendMail = $true,
[bool]$CreateDetailCSV = $true,
[string]$FileNamePrefix = "TLSReportMessageTrace",
[string]$LogFilePath = "$(Split-Path $MyInvocation.MyCommand.Path)",
[string]$LogFileFileNamePrefix = "TLSReport",
[string]$ExchangeOnlineURL = "https://outlook.office365.com/powershell-liveid",
[string]$ExOUsername = "svc-messagetrace@yourdomain.onmicrosoft.com",
[string]$ExOPassword = 'YourPassword',
[bool]$DisableDisplayLog = $true,
[int]$DeleteItemsOlderThanDays = 7
)



$global:DateTimeStamp = "$(Get-Date -Format yyyyMMddHHmm)"
$global:LogFile = "$($LogFilePath)\$($LogFileFileNamePrefix)_$($global:DateTimeStamp).log"

function Log {
Param(
[Parameter(Mandatory = $true)]
[string]$Message,
[Parameter(Mandatory = $true)]
[string]$Errorlevel,
[string]$LogFilePath = $global:LogFile,
[string]$ErrorLogFilePath = $ErrLogFilePath,
$NoLogging = $DisableDisplayLog
)
    switch ($ErrorLevel) {
        "Err" {$ErrLevelstr = "ERROR"}
        "Log" {$ErrLevelstr = "LOG"}
        "Warn" {$ErrLevelstr = "WARNING"}
    }
    $DateCode = "$(Get-Date -Format "yyyyMMdd-HHmm")"
    $LogMessage = "$($DateCode) - [$($ErrLevelstr)] $Message"
    if ($Errorlevel -eq "Err") {
        if ($ErrorLogFilePath) {
        "$LogMessage" | Add-Content -Path "$ErrorLogFilePath" -Force
        }
    }
    $LogMessage = "$($DateCode) [$($ErrLevelstr)] $message"
    if ($LogFilePath -ne "") {
        try {
            "$LogMessage" | Add-Content -Path "$($LogFilePath)" -Force -ErrorAction Stop
        } catch {
            throw "Cannot write Log $($LogFilePath)"
        }
    }
    if (!($NoLogging)) {
        if ($ErrorLevel -eq "Err") {
        Write-Host $LogMessage -ForegroundColor red
        } else {
            if ($ErrorLevel -eq "Warn") {
            Write-Host $LogMessage -ForeGroundColor yellow
            } else {
            Write-Host $LogMessage
            }
        }
    }
}


function Send-Mail {
Param(
[string]$MailServer = $MailServer,
[string]$MailServerPort = $MailServerPort,
[string]$MailServerUsername = $MailServerUsername,
[string]$MailServerPassword = $MailServerPassword,
[bool]$MailServerUseTLS = $MailServerUseTLS,
[bool]$MailServerIgnoreTLSValidation = $MailServerIgnoreTLSValidation,
[string]$MessageSubject = $MessageSubject,
[string]$MessageSender = $MessageSender,
[string]$MessageRecipient = $MessageRecipient,
[string]$MessageBody
)

try {
    $MailMessageObject = New-Object Net.Mail.MailMessage
    $MailObjectSmtp = New-Object Net.Mail.SmtpClient($MailServer, $MailServerPort)
	$MailMessageObject.From = "$MessageSender"
	if ($MailServerUsername -ne "" -or $MailServerUsername -ne $null) {
		Write-Verbose "Use Credentials to Authenticate against SMTP-Host with User `"$MailUser`""
		if ($MailServerPassword -ne "" -or $MailUserPassword -ne $null) {
			Write-Verbose "Using given Password to create Credential Object"
			$SMTPCredentials = New-Object System.Net.NetworkCredential($MailServerUsername, $($MailUserPassword))
			$MailObjectSmtp.Credentials = $SMTPCredentials
		}
	}
	if ($MailServerUseTLS -eq $true) {
		$MailObjectSmtp.EnableSsl = $true
		if ($MailServerIgnoreTLSValidation) {
			Log -Errorlevel Warn -Message "Certificate Validation is set to be ignored!"
			## Code From http://poshcode.org/624
			## Create a compilation environment
			$Provider=New-Object Microsoft.CSharp.CSharpCodeProvider
			$Compiler=$Provider.CreateCompiler()
			$Params=New-Object System.CodeDom.Compiler.CompilerParameters
			$Params.GenerateExecutable=$False
			$Params.GenerateInMemory=$True
			$Params.IncludeDebugInformation=$False
			$Params.ReferencedAssemblies.Add("System.DLL") | Out-Null
			$TASource=
@'
				 namespace Local.ToolkitExtensions.Net.CertificatePolicy{
					 public class TrustAll : System.Net.ICertificatePolicy {
						 public TrustAll() {}
						public bool CheckValidationResult(System.Net.ServicePoint sp,
							 System.Security.Cryptography.X509Certificates.X509Certificate cert, 
							 System.Net.WebRequest req, int problem) {return true;}
					}
				 } 
'@ 
			$TAResults=$Provider.CompileAssemblyFromSource($Params,$TASource)
			$TAAssembly=$TAResults.CompiledAssembly
			## We now create an instance of the TrustAll and attach it to the ServicePointManager
			$TrustAll=$TAAssembly.CreateInstance("Local.ToolkitExtensions.Net.CertificatePolicy.TrustAll")
			[System.Net.ServicePointManager]::CertificatePolicy=$TrustAll
			## end code from http://poshcode.org/624
			[System.Net.ServicePointManager]::ServerCertificateValidationCallback = {return $true}
		}
	}
    $MailMessageObject.ReplyTo = "$MessageSender"
    $MailMessageObject.To.Add("$MessageRecipient")
    $MailMessageObject.Subject = "$MessageSubject"
    $MailMessageObject.Body = "$MessageBody"
    $MailMessageObject.IsBodyHTML = $true
    $MailObjectSmtp.Send($MailMessageObject)
} catch {
    Log -Errorlevel Err -Message "Error while trying to send the TLS-Report mail to MessageRecipient. $($_.Exception.Message)"
}
}


Function CreateHTMLTable {
Param(
$HTMLObject
)

Foreach ($HTMLObjectItem in $HTMLObject) {

	if ($HTMLObjectItem.TLSProtocol -notmatch "TLS1_2") {
		if ($HTMLObjectItem.TLSProtocol -eq "None") {
			$backgroundcolor = 'style="background-color: red"'
		} else {
			$backgroundcolor = 'style="background-color: orange"'
		} 
	} else {
		$backgroundcolor = 'style="background-color: green"'
	}
	$HTMLrows += "<tr $backgroundcolor><td>$($HTMLObjectItem.Direction)</td><td>$($HTMLObjectItem.Domain)</td><td>$($HTMLObjectItem.TLSProtocol)</td><td>$($HTMLObjectItem.TLSEncryption)</td></tr>"
}
$tableheader = '<table><tr><td>Direction</td><td>Domain</td><td>TLS-Protocol</td><td>TLSEncryption</td></tr>'
$table = "$tableheader$HTMLrows</table>"
return $table
}

$ExOsecurePassword = ConvertTo-SecureString $ExOPassword -AsPlainText -Force
$ExOCredentials = New-Object System.Management.Automation.PSCredential($ExOUsername, $ExOsecurePassword)

$ExOPSSession = New-PSSession -ConfigurationName Microsoft.Exchange -ConnectionUri $ExchangeOnlineURL -AllowRedirect -Authentication Basic -Credential $ExOCredentials
if ($ExOPSSession) {
	Log -ErrorLevel Log -Message "Successfully connect Exchange Online Management Shell Session (URI: $($ExchangeOnlineURL); Username: $($ExOUsername))"
	Import-PSSession $ExOPSSession -AllowClobber -DisableNameChecking | Out-Null
} else {
	Log -ErrorLevel Err -Message "Failed to connect to Exchange Online Management Shell Session (URI: $($ExchangeOnlineURL); Username: $($ExOUsername)). $($_.Exception.Message)"
	Exit 1
}

$TodayDateStart = (Get-Date).AddDays([int]-$AmountofDaysToCheck) | Get-Date -Format "MM\/dd\/yyyy"
$TodayDateStart = "$($TodayDateStart) 00:00:00"
$TodayDateEnd = Get-Date -Format "MM\/dd\/yyyy"
$TodayDateEnd = "$($TodayDateEnd) 23:59:59"
Log -ErrorLevel Log -Message "Startdate: $TodayDateStart - Enddate: $TodayDateEnd"
$InternalDomainsRegExExpression = $InternalDomains -join "|"
#$AllMessages = Get-MessageTrace -StartDate "$TodayDateStart" -EndDate "$TodayDateEnd"

$MessageTracePageCounter = 1
$MessageTracePageMaximum = 1000
do {
	$AllMessagesPart,$AllMessagePartCount = $null
	$AllMessagesPart = Get-MessageTrace -StartDate "$($TodayDateStart)" -EndDate "$($TodayDateEnd)" -PageSize 5000 -Page $MessageTracePageCounter
	$AllMessagePartCount = ($AllMessagesPart | Measure-Object).Count
	if ($AllMessagesPart -ne $null) {
		$AllMessages += $AllMessagesPart
	}
	$MessageTracePageCounter++	
} 
while ($MessageTracePageCounter -le $MessageTracePageMaximum -and $AllMessagePartCount)

$AllIncomingMessages = $AllMessages | ? {$($_.SenderAddress) -notmatch $InternalDomainsRegExExpression}
$AllOutgoingMessages = $AllMessages | ? {$($_.RecipientAddress) -notmatch $InternalDomainsRegExExpression}

#$MatchMailHost = $($IncomingMessageDetails.Data) | Select-String -Pattern "OutboundProxyTargetHostname`" String=`"([A-Za-z0-9.]*)`"`/"

$MailHashArray = @()
Foreach ($IncomingMessage in $AllIncomingMessages) {
	$IncomingMessageDetails = $IncomingMessage | Get-MessageTraceDetail -Event Receive,SendExternal
	foreach ($IncomingMessageDetail in $IncomingMessageDetails) {
		$MailHash,$MatchMailHost,$MatchMailIP,$MatchMailHostname = $Null
		#$MatchMailHost = $($IncomingMessageDetails.Data) | Select-String -Pattern "OutboundProxyTargetHostname`" String=`"([A-Za-z0-9_.]*)`"`/"
		#$MatchMailHost = $($IncomingMessageDetails.Data) | Select-String -Pattern "S:ProxiedClientIPAddress=([0-9.]*);S:ProxiedClientHostname=([A-Za-z_0-9.]*)"

		if ($IncomingMessageDetail.Event -eq "Extern senden") {
			#the outgoing part of relayed messages is part of the messagedetail of the incoming message
			$Direction = "outgoing"
			$MatchMailIP = $($IncomingMessageDetail.Data) | Select-String -Pattern "OutboundProxyTargetIPAddress`" String=`"([0-9.]*)`""
			$MatchMailHost = $($IncomingMessageDetail.Data) | Select-String "OutboundProxyTargetHostname`" String=`"([A-Za-z0-9-_.]*)`""
			$Domain = ("$($IncomingMessage.RecipientAddress)" -Split "@")[1]
			try {
				$MatchMailIP = $MatchMailIP.Matches.Groups[1]
				$MatchMailHostname = $MatchMailHost.Matches.Groups[1]
			} catch {
				Log -Errorlevel Warn -Message "Mailhost cannot be retrieved (RegEx Issue). $($_.Exception.Message)"
			}
		} else {
			$Direction = "incoming"
			$MatchMailHost = $($IncomingMessageDetails.Data) | Select-String -Pattern "S:ProxiedClientIPAddress=([0-9.]*);S:ProxiedClientHostname=([A-Za-z0-9-_.]*)[;|`"]" -Allmatches
			$Domain	= ("$($IncomingMessage.SenderAddress)" -Split "@")[1]
			try {
				$MatchMailIP = $MatchMailHost.Matches.Groups[1]
				$MatchMailHostname = $MatchMailHost.Matches.Groups[2]
			} catch {
				Log -Errorlevel Warn -Message "Mailhost cannot be retrieved (RegEx Issue). $($_.Exception.Message)"
			}
		}
		if ($IncomingMessageDetail.Data -match "tlsversion=NONE") {
			$MailHash = @{
					Time		= "$($IncomingMessage.Received | Get-Date -Format "dd.MM.yyyy HH:mm:ss")"
					Recipient	= "$($IncomingMessage.RecipientAddress)"
					RecipientDomain = ("$($IncomingMessage.RecipientAddress)" -Split "@")[1]
					Sender		= "$($IncomingMessage.SenderAddress)"
					SenderDomain	= ("$($IncomingMessage.SenderAddress)" -Split "@")[1]
					Domain		= "$Domain"
					MailserverIP	= "$MatchMailIP"
					MailserverHost	= "$MatchMailHostname"
					Direction = "$Direction"
					TLSProtocol	= "None"
					TLSEncryption 	= "None"
					}
			$MailHashArray += New-Object PSObject -property $MailHash
		} elseif ($IncomingMessageDetail.Data -match "tlsversion") {
			$Match, $TLSProtocol, $TLSEncryption = $null
			$Match = $($IncomingMessageDetails.Data) | Select-String -Pattern "S:tlsversion=([A-Za-z_0-9]*);S:tlscipher=([A-Za-z_0-9]*)[;|`"]"
			try {
				$TLSProtocol = $Match.Matches.Groups[1]
				$TLSEncryption = $Match.Matches.Groups[2]

			} catch {
				"$($IncomingMessageDetails.Data)"
			}
			$MailHash = @{
					Time		= "$($IncomingMessage.Received | Get-Date -Format "dd.MM.yyyy HH:mm:ss")"
					Recipient	= "$($IncomingMessage.RecipientAddress)"
					RecipientDomain = ("$($IncomingMessage.RecipientAddress)" -Split "@")[1]
					Sender		= "$($IncomingMessage.SenderAddress)"
					SenderDomain	= ("$($IncomingMessage.SenderAddress)" -Split "@")[1] 
					Domain		= "$Domain"
					MailserverIP	= "$MatchMailIP"
					MailserverHost	= "$MatchMailHostname"
					Direction = "$Direction"
					TLSProtocol	= "$TLSProtocol"
					TLSEncryption 	= "$TLSEncryption"
					}
			$MailHashArray += New-Object PSObject -property $MailHash
		}
		
	}
}

$AllIncoming = $MailHashArray
$AllEntries = $AllIncoming
#$HTMLObject = $UniqueIncoming | Select @{Name="Domain"; Expression = {$_.SenderDomain}},@{Name="Direction"; Expression = {"incoming"}},TLSProtocol,TLSEncryption | Sort TLSProtocol
#$Filtered = $UniqueIncoming | Select Domain,Direction,TLSProtocol,TLSEncryption | Sort TLSProtocol

$MailHashArray = @()
Foreach ($OutgoingMessage in $AllOutgoingMessages) {
	$OutgoingMessageDetails = $OutgoingMessage | Get-MessageTraceDetail -Event SendExternal	
	$MatchMailIP = $($OutgoingMessageDetails.Data) | Select-String -Pattern "OutboundProxyTargetIPAddress`" String=`"([0-9.]*)`""
	$MatchMailHost = $($OutgoingMessageDetails.Data) | Select-String "OutboundProxyTargetHostname`" String=`"([A-Za-z0-9-_.]*)`""
	try {
		$MatchMailIP = $MatchMailIP.Matches.Groups[1]
		$MatchMailHostname = $MatchMailHost.Matches.Groups[1]
	} catch {
		Log -Errorlevel Warn -Message "Mailhost cannot be retrieved (RegEx Issue). $($_.Exception.Message)"
	}
	foreach ($OutgoingMessageDetail in $OutgoingMessageDetails) {
		$MailHash = $Null
		if ($OutgoingMessageDetail.Data -match "tlsversion=NONE") {
			$MailHash = @{
					Time		= "$($OutgoingMessage.Received | Get-Date -Format "dd.MM.yyyy HH:mm:ss")"
					Recipient	= "$($OutgoingMessage.RecipientAddress)"
					RecipientDomain = ("$($OutgoingMessage.RecipientAddress)" -Split "@")[1]
					Domain		= ("$($OutgoingMessage.RecipientAddress)" -Split "@")[1]
					MailserverIP	= "$MatchMailIP"
					MailserverHost	= "$MatchMailHostname"
					Direction	= "outgoing"
					Sender		= "$($OutgoingMessage.SenderAddress)"
					SenderDomain	= ("$($OutgoingMessage.SenderAddress)" -Split "@")[1] 
					TLSProtocol	= "None"
					TLSEncryption 	= "None"
					}
			$MailHashArray += New-Object PSObject -property $MailHash
		} elseif ($OutgoingMessageDetail.Data -match "tlsversion") {
			$Match, $TLSProtocol, $TLSEncryption = $null
			$Match = $($OutgoingMessageDetail.Data) | Select-String -Pattern "S:tlsversion=([A-Za-z_0-9]*);S:tlscipher=([A-Za-z_0-9]*)[;|`"]"
			try {
				$TLSProtocol = $Match.Matches.Groups[1]
				$TLSEncryption = $Match.Matches.Groups[2]
			} catch {
				"$($OutgoingMessageDetail.Data)"
			}
			$MailHash = @{
					Time		= "$($OutgoingMessage.Received | Get-Date -Format "dd.MM.yyyy HH:mm:ss")"
					Recipient	= "$($OutgoingMessage.RecipientAddress)"
					RecipientDomain = ("$($OutgoingMessage.RecipientAddress)" -Split "@")[1]
					Domain		= ("$($OutgoingMessage.RecipientAddress)" -Split "@")[1]
					MailserverIP	= "$MatchMailIP"
					MailserverHost	= "$MatchMailHostname"
					Direction	= "outgoing"
					Sender		= "$($OutgoingMessage.SenderAddress)"
					SenderDomain	= ("$($OutgoingMessage.SenderAddress)" -Split "@")[1] 
					TLSProtocol	= "$TLSProtocol"
					TLSEncryption 	= "$TLSEncryption"
					}
			$MailHashArray += New-Object PSObject -property $MailHash
		}
		
	}

}

try {
	Remove-PSSession $ExOPSSession -ErrorAction Stop
	Log -ErrorLevel Log -Message "Successfully removed Exchange Online Manegement Shell Session."
} catch {
	Log -ErrorLevel Err -Message "Error while removing Exchange Online Management Shell Session. $($_.Exception.Message)"
}


$AllOutgoing = $MailHashArray
$AllEntries += $AllOutgoing
#$AllFiltered = $AllEntries | Select Domain,Direction,TLSProtocol,TLSEncryption -Unique
#$AllFilteredEntries += $UniqueOutgoing | Select @{Name="Domain"; Expression = {$_.RecipientDomain}},@{Name="Direction"; Expression = {"outgoing"}},TLSProtocol,TLSEncryption | Sort TLSProtocol 
$AllFilteredEntries += $UniqueOutgoing | Select Domain,Direction,TLSProtocol,TLSEncryption | Sort TLSProtocol 

try {
	$AllEntries | Export-Csv -Path "$($LogFilePath)\$($FileNamePrefix)_$($global:DateTimeStamp).csv" -Encoding Default -delimiter ";" -NoTypeInformation
	Log -ErrorLevel Log -Message "Report successfully written to $($LogFilePath)\$($FileNamePrefix)_$($global:DateTimeStamp).csv."
} catch {
	Log -ErrorLevel Err -Message "Error while writing CSV-File $($LogFilePath)\$($FileNamePrefix)_$($global:DateTimeStamp).csv. $($_.Exception.Message)"
}	

$GroupedEntries = ($AllEntries | Group Domain,TLSProtocol,Direction,TLSEncryption) 
$TimeFilteredEntries = @()
Foreach ($GroupedEntry in $GroupedEntries) {
	$NewestEntry = $GroupedEntry.Group | Sort {[System.DateTime]::ParseExact($_.Time, "dd.MM.yyyy HH:mm:ss", $null)} -Descending | Select -First 1
	$TimeFilteredEntries += $NewestEntry
}

$AllFiltered = $TimeFilteredEntries 

$AllUniqueDomains = $AllFiltered | Select Domain -Unique

$ConsolidatedDomainList = @()
foreach ($UniqueDomain in $AllUniqueDomains) {
	Log -ErrorLevel Log -Message "Checking for None TLSEncryption coming from Domain $($UniqueDomain.Domain)"
	$IncomingEntries = $AllFiltered | ? {$($_.Domain) -eq "$($UniqueDomain.Domain)" -and ($_.Direction -eq "incoming") -and ($_.TLSEncryption -eq "None")}
	if ($IncomingEntries) {
		$ConsolidatedDomainList += $IncomingEntries | select -First 1
		Log -ErrorLevel Warn -Message "At least one connection was not secured from Domain $($UniqueDomain.Domain)"
	} else {
		$ConsolidatedDomainList += $AllFiltered | ? {$_.Direction -eq "incoming" -and ($_.Domain -eq ($UniqueDomain.Domain))}
		Log -ErrorLevel Log -Message "All connections from $($UniqueDomain.Domain) are secured"
	}
	Log -ErrorLevel Log -Message "Checking for None TLSEncryption to Domain $($UniqueDomain.Domain)"
	$OutgoingEntries = $AllFiltered | ? {$($_.Domain) -eq "$($UniqueDomain.Domain)" -and ($_.Direction -eq "outgoing") -and ($_.TLSEncryption -eq "None")}
	if ($OutgoingEntries) {
		$ConsolidatedDomainList += $OutgoingEntries | select -First 1
		Log -ErrorLevel Warn -Message "At least one connection was not secured to Domain $($UniqueDomain.Domain)"
	} else {
		$ConsolidatedDomainList += $AllFiltered | ? {$_.Direction -eq "outgoing" -and ($_.Domain -eq ($UniqueDomain.Domain))}
		Log -ErrorLevel Log -Message "All connections to $($UniqueDomain.Domain) are secured"
	}
}

if ($ConsolidatedDomainList) {
	try {
		$ConsolidatedDomainList = $ConsolidatedDomainList | Sort TLSProtocol,Domain
		$HTMLFile += CreateHTMLTable -HTMLObject $ConsolidatedDomainList
		$HTMLFile | Out-File "$($LogFilePath)\$($FileNamePrefix)_$($global:DateTimeStamp).html" -Force -ErrorAction Stop
		Log -ErrorLevel Log -Message "Successfull saved HTML-Report to $($LogFilePath)\$($FileNamePrefix)_$($global:DateTimeStamp).html"
	} catch {
		Log -ErrorLevel Err -Message "Error while writing HTML-Report to $($LogFilePath)\$($FileNamePrefix)_$($global:DateTimeStamp).html. $($_.Exception.Message)"
	}
} else {
	$HTMLFile = "No Message traces existent for the defined time period"
}


if ($SendMail) {
	Log -ErrorLevel Log -Message "Trying to send mail to $($MessageRecipient)"
	Send-Mail -MessageBody $HTMLFile -MessageSubject $MessageSubject
}


if ($DeleteItemsOlderThanDays -gt 0) {
	if (Test-Path ($LogFilePath)) {
		$LastWrite = (Get-Date).AddDays(-$DeleteItemsOlderThanDays)
		Log -ErrorLevel Log -Message "Cleanup old Log- and Trace-files in folder $($LogFilePath) older than $($LastWrite)"
		$FilesToBeDeleted = Get-ChildItem -Path "$($LogFilePath)\*" -File -Include "$($FileNamePrefix)_*.csv","$($FileNamePrefix)_*.html","$($LogFileFileNamePrefix)_*.log"  | Where {$_.LastWriteTime -le "$LastWrite"}
		$FilesToBeDeleted | Remove-Item -ErrorAction Stop | out-null
	}
}
